package com.persagy.dmp.middleware.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.persagy.dmp.middleware.dao.WdConstructDescriptionMapper;
import com.persagy.dmp.middleware.entity.dto.InstanceUrlParam;
import com.persagy.dmp.middleware.entity.response.CommonResult;
import com.persagy.dmp.middleware.entity.vo.WdConstructDescription;
import com.persagy.dmp.middleware.service.IConstructDescriptionService;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @ClassName ConstructDescriptionServiceImpl
 * @Description: 土建装饰说明
 * @Author linhuili
 * @Date 2021/9/14 15:27
 * @Version V1.0
 **/
@Service
public class ConstructDescriptionServiceImpl extends ServiceImpl<WdConstructDescriptionMapper, WdConstructDescription>  implements IConstructDescriptionService {

    @Autowired
    private WdConstructDescriptionMapper constructDescriptionMapper;

    /**
     * 查询土建装饰说明
     * @param context
     * @param vo
     * @return
     */
    @Override
    public CommonResult query(InstanceUrlParam context, WdConstructDescription vo) {
        QueryWrapper<WdConstructDescription> wrapper = new QueryWrapper<>();
        if(vo.getId() != null){
            wrapper.eq("id",vo.getId());
        }
        if(StringUtils.isNotEmpty(vo.getFloorId())){
            wrapper.eq("floor_id",vo.getFloorId());
        }
        if(StringUtils.isNotEmpty(vo.getWdSystemCode())){
            wrapper.eq("wd_system_code",vo.getWdSystemCode());
        }
        List<WdConstructDescription> wdConstructDescriptions = constructDescriptionMapper.selectList(wrapper);
        CommonResult<List<WdConstructDescription>> response = CommonResult.success(wdConstructDescriptions);
        Long total = response.getCount() == null ? null : response.getCount().longValue();
        response.setCount(total);
        return response;
    }

    /**
     * 新增土建装饰说明
     * @param context
     * @param vos
     * @return
     */
    @Override
    public CommonResult insert(InstanceUrlParam context, List<WdConstructDescription> vos) {
        saveOrUpdateBatch(vos);
        return CommonResult.success(vos);
    }

    /**
     * 修改土建装饰说明
     * @param context
     * @param vos
     * @return
     */
    @Override
    public CommonResult update(InstanceUrlParam context, List<WdConstructDescription> vos) {
        saveOrUpdateBatch(vos);
        return CommonResult.success(vos);
    }

    /**
     * 删除土建装饰说明
     * @param context
     * @param vos
     * @return
     */
    @Override
    public CommonResult delete(InstanceUrlParam context, List<String> vos) {
        constructDescriptionMapper.deleteBatchIds(vos);
        return CommonResult.success(vos);
    }

}
