package com.persagy.dmp.middleware.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.node.ArrayNode;
import com.fasterxml.jackson.databind.node.JsonNodeFactory;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.dmp.basic.model.QueryCriteria;
import com.persagy.dmp.digital.entity.ObjectRelation;
import com.persagy.dmp.middleware.constant.CommonConsant;
import com.persagy.dmp.middleware.constant.enums.RelationTypeEnum;
import com.persagy.dmp.middleware.dao.WdConstructDescriptionMapper;
import com.persagy.dmp.middleware.dao.WdDefineTypeRelMapper;
import com.persagy.dmp.middleware.entity.dto.CommonRequestDTO;
import com.persagy.dmp.middleware.entity.dto.InstanceUrlParam;
import com.persagy.dmp.middleware.entity.response.CommonResult;
import com.persagy.dmp.middleware.entity.vo.WdConstructDescription;
import com.persagy.dmp.middleware.entity.vo.WdDefineTypeRel;
import com.persagy.dmp.middleware.service.IBaseService;
import com.persagy.dmp.middleware.service.IEquipService;
import com.persagy.dmp.middleware.service.ISpaceService;
import com.persagy.dmp.middleware.utils.EntityTransferUtil;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @ClassName ObjectInstanceServiceImpl
 * @Description: 广联达：查询设备，空间，阀门
 * @Author linhuili
 * @Date 2021/9/10 11:17
 * @Version V1.0
 **/
@Service
public class EquipServiceImpl implements IEquipService {

    @Autowired
    private WdDefineTypeRelMapper defineTypeRelMapper;

    @Autowired
    private IBaseService baseService;

    @Autowired
    private ISpaceService spaceService;

    @Autowired
    private WdConstructDescriptionMapper constructDescriptionMapper;

    /**
     * 查询设备设施模块下的设备/阀门
     * @param context
     * @param request
     * @return
     */
    @Override
    public CommonResult query(InstanceUrlParam context, CommonRequestDTO request) {

        List<JSONObject> result = new ArrayList<>();

        //查询系统下的设备
        List<JSONObject> equipList = querySystemEquipList(context, request);
        if(CollUtil.isNotEmpty(equipList)){
            result.addAll(equipList);
        }

        //查询系统下的阀门
        List<JSONObject> materiaList = querySystemMateriaList(context, request);
        if(CollUtil.isNotEmpty(materiaList)){
            result.addAll(materiaList);
        }

        return CommonResult.success(result);
    }


    /**
     * 查询设备设施-暖通系统-全局-3D原理图相关的设备/阀门
     * @param context
     * @param request
     * @return
     */
    @Override
    public CommonResult queryHavcSystem(InstanceUrlParam context, CommonRequestDTO request) throws Exception {
        // 查询冷却塔设备类下的设备，制冷机房空间下的设备/阀门信息
        List<JSONObject> result = new ArrayList<>();

        //1.查询冷却塔设备类下的设备/阀门
        List<WdDefineTypeRel> coolTowerDefineTypeRels = queryGldHavcTypeRel(CommonConsant.ACCCOT_BIMTYPEID,CommonConsant.WD_HAVC_SYSTEM_CODE);
        List<JSONObject> coolTowerObjects = queryEquipObjectByInfoCode(context, coolTowerDefineTypeRels, request, CommonConsant.BIM_TYPE_ID);
        if(CollUtil.isNotEmpty(coolTowerObjects)){
            result.addAll(coolTowerObjects);
        }

        //2. 查询制冷机房空间下的所有空间
        QueryCriteria queryCriteria = new QueryCriteria();
        ObjectNode objectNode = JsonNodeFactory.instance.objectNode();
        objectNode.put(CommonConsant.BIM_TYPE_ID,CommonConsant.ACCCOT_SPACE_BIMTYPEID);
        queryCriteria.setCriteria(objectNode);
        CommonResult commonResult = baseService.doQuery(context, queryCriteria, JSONObject.class);
        List<JSONObject> spaceList = (List<JSONObject>) commonResult.getContent();

        if(CollUtil.isEmpty(spaceList)){
            return CommonResult.success(result);
        }

        //3.查询空间下的设备/阀门
        Set<String> spaceIds = spaceList.stream().map(k->k.getString("id")).collect(Collectors.toSet());
        request.setSpaceIds(spaceIds);
        List<JSONObject> content = spaceService.queryEquipInSpace(context, request, CommonConsant.ACCCOT_SPACE_BIMTYPEID);
        if(CollUtil.isNotEmpty(content)){
            result.addAll(content);
        }

        return CommonResult.success(result);
    }

    /**
     * 设备设施-暖通系统-分集水器原理图查询
     * @param context
     * @param request
     * @return
     * @throws Exception
     */
    @Override
    public CommonResult querysubCatchment(InstanceUrlParam context, CommonRequestDTO request) throws Exception {
        // 查询分水器/集水器下的对象信息
        List<JSONObject> result = new ArrayList<>();

        //1.查询分水器
        List<WdDefineTypeRel> divideDefineTypeRels = queryGldHavcTypeRel(CommonConsant.DIVIDE_BREECH_BIMTYPEID,CommonConsant.WD_HAVC_SYSTEM_CODE);
        List<JSONObject> divideBreechObjects = queryEquipObjectByInfoCode(context, divideDefineTypeRels, request, CommonConsant.BIM_TYPE_ID);
        if(CollUtil.isNotEmpty(divideBreechObjects)){
            result.addAll(divideBreechObjects);
        }
        //2. 查询集水器
        List<WdDefineTypeRel> collectDefineTypeRels = queryGldHavcTypeRel(CommonConsant.COLLECT_BREECH_BIMTYPEID,CommonConsant.WD_HAVC_SYSTEM_CODE);
        List<JSONObject> collectBreechObjects = queryEquipObjectByInfoCode(context, collectDefineTypeRels, request, CommonConsant.BIM_TYPE_ID);
        if(CollUtil.isNotEmpty(collectBreechObjects)){
            result.addAll(collectBreechObjects);
        }
        return CommonResult.success(result);
    }

    /**
     * 统计防火分区数量
     * @param context
     * @param request
     * @return
     * @throws Exception
     */
    @Override
    public CommonResult countFireZone(InstanceUrlParam context, CommonRequestDTO request) throws Exception {
        return null;
    }


    /**
     * 获取系统下的设备
     * @param context
     * @param request
     */
    private List<JSONObject>  querySystemEquipList(InstanceUrlParam context, CommonRequestDTO request){
        //查询广联达设备类型
        List<WdDefineTypeRel> defineTypeRels = queryGldEquipTypeRel();
        //根据系统过滤
        if(StringUtils.isNotEmpty(request.getSystemCode())){
            String systemCode = request.getSystemCode();
            defineTypeRels = defineTypeRels.stream().filter(k->!systemCode.equals(k.getWdSystemCode())).collect(Collectors.toList());
        }
        //根据广联达设备类型查询设备信息
        List<JSONObject> equipList = queryEquipByGldTypeRel(context, defineTypeRels, request);
        return equipList;
    }


    /**
     * 获取系统下的阀门
     * @param context
     * @param request
     */
    private List<JSONObject>  querySystemMateriaList(InstanceUrlParam context, CommonRequestDTO request){
        //查询广联达设备类型
        List<WdDefineTypeRel> defineTypeRels = queryGldMateriaTypeRel();
        //根据系统过滤
        if(StringUtils.isNotEmpty(request.getSystemCode())){
            String systemCode = request.getSystemCode();
            defineTypeRels = defineTypeRels.stream().filter(k->!systemCode.equals(k.getWdSystemCode())).collect(Collectors.toList());
        }
        //根据广联达设备类型查询设备信息
        List<JSONObject> equipList = queryMateriaByGldTypeRel(context, defineTypeRels, request);
        return equipList;
    }


    /**
     * 查询万达映射表中的设备类型
     */
    @Override
    public List<WdDefineTypeRel> queryGldEquipTypeRel(){
        QueryWrapper<WdDefineTypeRel> wrapper = new QueryWrapper<>();
        wrapper.eq("obj_type","equipment");
        List<WdDefineTypeRel> defineTypeRels = defineTypeRelMapper.selectList(wrapper);
        return defineTypeRels;
    }


    /**
     * 根据广联达设备类型查询设备信息
     * @param context
     * @param defineTypeRels
     * @param request
     */
    @Override
    public List<JSONObject> queryEquipByGldTypeRel(InstanceUrlParam context, List<WdDefineTypeRel> defineTypeRels, CommonRequestDTO request){
        // 根据bimTypeId查询楼层下的设备
        List<JSONObject> bimTypeSpace = queryEquipObjectByInfoCode(context, defineTypeRels, request, "bimTypeId");
        // 数据组装：根据bimTypeId查询楼层下的设备结果
        List<JSONObject> result = combineQueryEquipResultData(defineTypeRels, bimTypeSpace, "bimTypeId");
        return result;
    }


    /**
     * 根据信息点查询楼层下的设备对象信息
     * @param context
     * @param defineTypeRels
     * @param request
     * @return
     */
    @Override
    public List<JSONObject> queryEquipObjectByInfoCode(InstanceUrlParam context, List<WdDefineTypeRel> defineTypeRels, CommonRequestDTO request, String infoCode){
        //1. 根据bimTypeId查询楼层下的设备
        Set<String> bimTypeIdSets = defineTypeRels.stream().filter(k-> StringUtils.isNotEmpty(k.getBimTypeId())).map(k->k.getBimTypeId()).collect(Collectors.toSet());
        ObjectNode criteria = JsonNodeFactory.instance.objectNode();
        ArrayNode bimTypeIds = criteria.withArray(infoCode);
        bimTypeIdSets.forEach(item->{bimTypeIds.add(item);});

        QueryCriteria queryCriteria = new QueryCriteria();
        //根据建筑/楼层id关系查询
        if(StringUtils.isNotEmpty(request.getFloorId())){
            queryCriteria = EntityTransferUtil.transDmpRelationCriteria(criteria, RelationTypeEnum.MECHINARCH_EQ2FL.getGraphCode(),
                            RelationTypeEnum.MECHINARCH_EQ2FL.getRelCode(), request.getFloorId(), 1);
        }else{
            queryCriteria = EntityTransferUtil.transDmpRelationCriteria(criteria, RelationTypeEnum.MECHINARCH_EQ2BD.getGraphCode(),
                            RelationTypeEnum.MECHINARCH_EQ2BD.getRelCode(), request.getBuildId(), 1);
        }

        CommonResult commonResult = baseService.doQuery(context, queryCriteria, JsonNode.class);
        return (List<JSONObject>)commonResult.getContent();
    }


    /**
     * 查询万达映射表中的阀门类型
     */
    @Override
    public List<WdDefineTypeRel> queryGldMateriaTypeRel(){
        QueryWrapper<WdDefineTypeRel> wrapper = new QueryWrapper<>();
        wrapper.eq("obj_type","material");
        List<WdDefineTypeRel> defineTypeRels = defineTypeRelMapper.selectList(wrapper);
        return defineTypeRels;
    }


    /**
     * 根据广联达阀门类型查询阀门信息
     * @param context
     * @param defineTypeRels
     * @param request
     */
    @Override
    public List<JSONObject> queryMateriaByGldTypeRel(InstanceUrlParam context, List<WdDefineTypeRel> defineTypeRels, CommonRequestDTO request){
        // 根据bimTypeId查询楼层下的设备
        List<JSONObject> bimTypeSpace = queryEquipObjectByInfoCode(context, defineTypeRels, request,"bimTypeId");
        // 数据组装：根据bimTypeId查询楼层下的设备结果
        List<JSONObject> result = combineQueryEquipResultData(defineTypeRels, bimTypeSpace, "bimTypeId");
        return result;
    }


    /**
     * 组装查询设备结果
     * @param defineTypeRels
     * @param spaceList
     * @param infoCode
     * @return
     */
    private List<JSONObject>  combineQueryEquipResultData(List<WdDefineTypeRel> defineTypeRels, List<JSONObject> spaceList, String infoCode){
        if(CollUtil.isEmpty(defineTypeRels)){
            return new ArrayList<>();
        }
        Map<String, List<JSONObject>> spaceMap = spaceList.stream().collect(Collectors.groupingBy(k->k.getString(infoCode)));

        //数据组装
        List<JSONObject> result = new ArrayList<>();
        for (WdDefineTypeRel defineTypeRel : defineTypeRels) {
            JSONObject data = new JSONObject();
            data.put("bimTypeId",defineTypeRel.getBimTypeId());
            data.put("className",defineTypeRel.getWdClassName());
            data.put("systemCode",defineTypeRel.getWdSystemCode());

            List<JSONObject> objects = new ArrayList<>();
            if(CollUtil.isNotEmpty(spaceMap)){
                spaceMap.get(defineTypeRel.getBimTypeId());
            }
            data.put("objects",objects);
            data.put("count",objects.size());
            result.add(data);
        }
        return result;

    }

    /**
     * 条件查询广联达映射关系
     * @param bimTypeId
     * @param systemCode
     * @return
     */
    public List<WdDefineTypeRel> queryGldHavcTypeRel(String bimTypeId, String systemCode){
        QueryWrapper<WdDefineTypeRel> wrapper = new QueryWrapper<>();

        wrapper.eq("bim_type_id",bimTypeId);
        wrapper.eq("wd_system_code",systemCode);
        List<WdDefineTypeRel> defineTypeRels = defineTypeRelMapper.selectList(wrapper);
        return defineTypeRels;
    }

}
